from django.urls import path
from . import views

app_name = "store"

urlpatterns = [
    path("database/", views.database_update, name="database_update"),
    path("database/clear/", views.clear_database_but_images, name="clear_database_but_images"),
    path("database/clear/images/", views.clear_database_images, name="clear_database_images"),

    path('', views.homepage, name="homepage"),
    path('search/', views.search, name="search"),
    path("apply-discount/", views.apply_discount),
    path("products/get-variations/", views.get_variations, name="get_variations"),
    path("products/add-to-cart/", views.add2cart, name="add2cart"),
    path("products/<slug:product_slug>/", views.product, name="product"),
    path("products/<slug:product_slug>/<int:variation_id>/", views.variation, name="variation"),
    path("cart/", views.getCart),
    path("cart/add-delivery-address/", views.addDeliveryAddress2Cart),
    path("cart/add-new-delivery-address/", views.addNewDeliveryAddress2Cart),
    path("cart/remove-delivery-address/", views.removeSelectedDeliveryAddress2Cart),
    path("cart/clear/", views.clearCart),
    path("cart/remove-item/", views.removeFromCart),
    path("checkout/", views.checkout, name="checkout"),
    path("checkout/place-order/", views.place_order),
    path('get-user-address/', views.getUserAddress),
    path('orders/', views.orders, name="orders"),
    path('orders/cancel/', views.cancelOrder),
    path('orders/re-open/<int:order_id>/', views.reopenOrder, name="reopenOrder"),
    path('orders/<int:order_id>/', views.order, name="order"),
    path("<slug:category_slug>/", views.homepage, name="products_by_category"),
]



from django.contrib import admin
from . import models
from django.db.models import Count
from django.utils.html import format_html, urlencode
from django.urls import reverse


@admin.register(models.Order)
class OrderAdmin(admin.ModelAdmin):
    pass

@admin.register(models.OrderItem)
class OrderItemAdmin(admin.ModelAdmin):
    pass

@admin.register(models.OrderCancellation)
class OrderCancellationAdmin(admin.ModelAdmin):
    pass


@admin.register(models.Image)
class ImageAdmin(admin.ModelAdmin):
    list_display = ['image', 'name']
    list_editable = ['name']
    list_per_page = 10
    search_fields = ['name']


@admin.register(models.Tag)
class TagAdmin(admin.ModelAdmin):
    list_display = ['id', 'name']
    list_editable = ['name']
    ordering = ['name']
    search_fields = ['name']


@admin.register(models.Category)
class CategoryAdmin(admin.ModelAdmin):
    list_display = ['id', 'title', 'product_count']
    list_per_page = 10
    ordering = ['title']
    search_fields = ['title']
    prepopulated_fields = {
        'slug': ['title']
    }

    @admin.display(ordering='product_count')
    def product_count(self, category):
        url = (
            reverse('admin:store_product_changelist')
            + '?'
            + urlencode({
                'categories__id': str(category.id)
            })
        )
        return format_html(
            '<a href="{}">{}</a>',
            url,
            category.product_count
        )
    
    def get_queryset(self, request):
        return super().get_queryset(request).annotate(
            product_count=Count('products')
        )


class ProductVariationInline(admin.StackedInline):
    model = models.Variation
    min_num = 1
    extra = 0
    autocomplete_fields = ['product', 'images', 'keyvalues']


@admin.register(models.Product)
class ProductAdmin(admin.ModelAdmin):
    autocomplete_fields = ['images', 'tags', 'categories']
    inlines = [ProductVariationInline]
    list_display = ['title']
    list_per_page = 10
    list_filter = ['categories__title']
    search_fields = ['title__istartswith']
    prepopulated_fields = {
        'slug': ['title']
    }


@admin.register(models.Key)
class KeyAdmin(admin.ModelAdmin):
    list_display = ['key', 'value_count']
    search_fields = ['key']

    @admin.display(ordering='value_count')
    def value_count(self, key):
        url = (
            reverse('admin:store_value_changelist')
            + '?'
            + urlencode({
                'key__id': str(key.id)
            })
        )
        return format_html(
            '<a href="{}">{}</a>',
            url,
            key.value_count
        )
    
    def get_queryset(self, request):
        return super().get_queryset(request).annotate(
            value_count=Count('values')
        )


@admin.register(models.Value)
class ValueAdmin(admin.ModelAdmin):
    search_fields = ['value']


class InventoryFilter(admin.SimpleListFilter):
    title = 'inventory'
    parameter_name = 'inventory'

    def lookups(self, request, model_admin):
        return [
            ('<10', "Low")
        ]
    
    def queryset(self, request, queryset):
        if self.value() == '<10':
            return queryset.filter(inventory__lt=10)


@admin.register(models.Variation)
class VariationAdmin(admin.ModelAdmin):
    autocomplete_fields = ['product', 'images', 'keyvalues']
    list_display = ['product', 'price', 'unit', 'availability', 'inventory', 'inventory_status']
    list_editable = ['price', 'availability', 'inventory']
    list_per_page = 10
    list_filter = [InventoryFilter, 'keyvalues__value']
    search_fields = ['product__title__istartswith']

    @admin.display(ordering='inventory')
    def inventory_status(self, variation):
        if variation.inventory < 10:
            return 'Low'
        return 'OK'


@admin.register(models.Discount)
class DiscountAdmin(admin.ModelAdmin):
    actions = ['clear_discounts']
    list_display = ['id', 'code', 'percent']
    list_editable = ['code', 'percent']

    @admin.action(description='Clead Discounts')
    def clear_discounts(self, request, queryset):
        updated_count = queryset.update(percent=0)
        self.message_user(
            request,
            f'{updated_count} discounts were successfully updated.'
        )


@admin.register(models.KeyValue)
class KeyValueAdmin(admin.ModelAdmin):
    autocomplete_fields = ['key', 'value']
    list_display = ['id', 'key_title', 'value_title', 'product_count']
    # list_editable = ['key_title', 'value_title']
    list_select_related = ['key', 'value']
    search_fields = ['key__key__istartswith', 'value__value__istartswith']

    @admin.display(ordering='key')
    def key_title(self, keyvalue):
        return keyvalue.key.key
    
    @admin.display(ordering='value')
    def value_title(self, keyvalue):
        return keyvalue.value.value
    
    @admin.display(ordering='product_count')
    def product_count(self, keyvalue):
        url = (
            reverse('admin:store_variation_changelist')
            + '?'
            + urlencode({
                'keyvalues__id': str(keyvalue.id)
            })
        )
        return format_html(
            '<a href="{}">{}</a>',
            url,
            keyvalue.product_count
        )
    
    def get_queryset(self, request):
        return super().get_queryset(request).annotate(
            product_count=Count('variation')
        )


from .models import Order

def get_order_details(request, order_id):
    try:
        order_id = int(order_id)
        order = Order.objects.select_related("address", "discount", "cancellation").get(id=order_id)
    except:
        return None
    if order.customer != request.user.person:
        return None
    
    items = order.orderitems.select_related().all()
    i = []
    for item in items:
        k_v = []
        keyvalues = item.product.keyvalues.select_related("key", "value").all()
        for kv in keyvalues:
            k_v.append({
                "key": kv.key.key.strip().lower().title(),
                "value": kv.value.value.strip().lower().title(),
            })

        i.append({
            "id": item.id,
            "product": {
                "product_title": item.product.product.title.strip().lower().title(),
                "product_slug": item.product.product.slug,
                "variation_id": item.product.id,
                "unit": item.product.unit.strip().lower().title(),
                "price": round(float(item.product.price), 2),
                "keyvalues": k_v
            },
            "qty": item.quantity,
            "unit_price": item.unit_price,
            "amount": round(float(item.amount), 2)
        })
    
    if order.status == 'P':
        cancellation = {
            "cancelled_at": order.cancellation.cancelled_at,
            "reason": order.cancellation.reason
        }
    else:
        cancellation = None

    o = {
        "id": order.id,
        "address": {
            "id": order.address.id,
            "first_name": order.address.first_name,
            "last_name": order.address.last_name,
            "email": order.address.email,
            "mobile": order.address.mobile,
            "homephone": order.address.homephone,
            "address1": order.address.address1,
            "address2": order.address.address2,
            "landmark": order.address.landmark,
            "pincode": order.address.pincode,
            "city": order.address.city,
            "state": order.address.state,
            "country": order.address.country
        },
        "amount": order.amount,
        "discount": None if not order.discount else order.discount.percent,
        "amount_payable": order.amount_payable,
        "status": order.status,
        "placed_at": order.placed_at,
        "closed_at": order.closed_at,
        "cancelled_at": order.cancelled_at,
        "preferred_delivery_time": order.preferred_delivery_time,
        "additional_info": order.additional_info,
        "count": order.count,
        "items": i,
        "cancellation": cancellation
    }

    return o


def validate_email(emailaddress):
    pos_AT = 0
    count_AT = 0
    count_DT = 0
    if emailaddress[0] == '@' or emailaddress[-1] == '@':
        return False
    if emailaddress[0] == '.' or emailaddress[-1] == '.':
        return False
    for c in range(len(emailaddress)):
        if emailaddress[c] == '@':
            pos_AT = c
            count_AT = count_AT + 1
    if count_AT != 1:
        return False
        
    username = emailaddress[0:pos_AT]
    if not username[0].isalnum() or not username[-1].isalnum():
        return False
    for d in range(len(emailaddress)):
        if emailaddress[d] == '.':
            if d == (pos_AT+1):
                return False
            if d > pos_AT:
                word = emailaddress[(pos_AT+1):d]
                if not word.isalnum():
                    return False
                pos_AT = d
                count_DT = count_DT + 1
    if count_DT < 1 or count_DT > 2:
        return False
        
    return True




from .models import Category
from django.conf import settings

def site_defaults(request):
    categories = Category.objects.all()
    try:
        sitetitle = settings.SITE_TITLE
    except AttributeError:
        sitetitle = "blackparis"
    
    try:
        currency = settings.CURRENCY
    except AttributeError:
        currency = "Rs."
    
    try:
        delivery_time_hours = settings.DELIVERY_TIME_IN_HOURS
    except AttributeError:
        delivery_time_hours = 24
    
    try:
        delivery_time_days = settings.DELIVERY_TIME_IN_DAYS
    except AttributeError:
        delivery_time_days = 1
    
    if request.session.get("cart", False) and request.session["cart"].get("cart_items", False):
        count = len(request.session["cart"]["cart_items"].keys())
    else:
        count = 0

    context = {
        "categories": categories,
        "sitetitle": sitetitle,
        "currency": currency,
        "cartItemsCount": count,
        "delivery_time_hours": delivery_time_hours,
        "delivery_time_days": delivery_time_days
    }
    return context


from django.core.paginator import Paginator
from .models import Product, Category, Tag


def get_products(request, category_slug):
    if category_slug:
        try:
            c = Category.objects.get(slug=category_slug)
        except:
            return None
        else:
            products = c.products.prefetch_related("images", "variations").all()
    else:
        products = Product.objects.prefetch_related("images", "variations").all()
    
    paginator = Paginator(products, 12)
    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)
    return page_obj


def get_orders(request):
    orders = request.user.person.orders.select_related("address", "cancellation", "discount").all()
    paginator = Paginator(orders, 10)
    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)
    return page_obj


def get_search_products(request, q):
    products = Product.objects.prefetch_related("images", "variations").filter(title__istartswith=q)
    tags = Tag.objects.prefetch_related("products").filter(name__istartswith=q)
    for tag in tags:
        products = products.union(tag.products.prefetch_related("images", "variations").all()).order_by('title')
    paginator = Paginator(products, 12)
    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)
    return page_obj


from django.shortcuts import render
from django.http import HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.views.decorators.csrf import ensure_csrf_cookie
from django.db import IntegrityError, transaction
from time import sleep
from datetime import datetime, timezone
import math

from store.models import Discount, Product, Variation, Order, OrderItem, OrderCancellation
from authentication.models import Address
from . import database
from .pagination import get_products, get_orders, get_search_products
from .utility import validate_email, get_order_details


@ensure_csrf_cookie
def homepage(request, category_slug=None):
    products = get_products(request, category_slug)
    if products == None:
        return HttpResponseRedirect(reverse("store:homepage"))
    return render(
        request,
        "store/homepage.html",
        {"products": products}
    )


def product(request, product_slug):
    try:
        product_detail = Product.objects.prefetch_related(
            "images",
            "categories",
            "variations",
            "variations__keyvalues",
            "variations__keyvalues__key",
            "variations__keyvalues__value",
        ).get(slug=product_slug)
    except:
        return HttpResponseRedirect(reverse("store:homepage"))
    
    keys = set()
    variations = product_detail.variations.all()
    for variation in variations:
        for kvs in variation.keyvalues.all():
            keys.add(kvs.key.key)
    
    return render(
        request,
        "store/product.html",
        {"product": product_detail, "keys": keys}
    )


def variation(request, product_slug, variation_id):
    try:
        variation = Variation.objects.select_related("product").prefetch_related("keyvalues", "images", "keyvalues__key", "keyvalues__value").get(id=variation_id)
    except:
        return HttpResponseRedirect(reverse("store:homepage"))
    return render(
        request,
        "store/variation.html",
        {"variation": variation}
    )


def get_variations(request):
    if request.method == "GET":
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    product_id = request.POST["product_id"]
    if not product_id:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        product_id = int(product_id)
    except ValueError:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        product = Product.objects.get(id=product_id)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        currency = settings.CURRENCY
    except AttributeError:
        currency = "Rs."
    
    variations = product.variations.prefetch_related("keyvalues", "images").all()
    v = []
    for variation in variations:
        if variation.availability and variation.inventory >= 1:
            i = []
            images = variation.images.all()[1:]
            for image in images:
                i.append({
                    "image": image.image.url,
                    "name": image.name
                })
            
            k_v = []
            keyvalues = variation.keyvalues.select_related("key", "value").all()
            for kv in keyvalues:
                k_v.append({
                    "key": kv.key.key.strip().lower().title(),
                    "value": kv.value.value.strip().lower().title()
                })
            
            v.append({
                "id": variation.id,
                "title": product.title.strip().lower().title(),
                "variation_url": reverse("store:variation", kwargs={"product_slug": product.slug, "variation_id": variation.id}),
                "price": round(float(variation.price), 2),
                "currency": currency,
                "unit": variation.unit.strip().lower().title(),
                "short_info": ((variation.info)[:200] + "..."),
                "info": variation.info,
                "keyvalues": k_v,
                "first_image_url": variation.images.first().image.url,
                "first_image_name": variation.images.first().name,
                "all_images": i,
            })
    return JsonResponse({"success": True, "variations": v, "product_name": product.title.strip().lower().title()})


def add2cart(request):
    if request.method == "GET":
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    var_id = request.POST["var_id"]
    qty = request.POST["qty"]

    if not var_id or not qty:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        var_id = int(var_id)
        qty = float(qty)
    except ValueError:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if qty <= 0 or not Variation.objects.filter(id=var_id).exists():
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if not request.session.get("cart", False):
        request.session["cart"] = {"cart_items": {}}
        request.session.modified = True
    
    varid = str(var_id)
    if varid not in request.session["cart"]["cart_items"]:
        request.session["cart"]["cart_items"][varid] = qty
    else:
        request.session["cart"]["cart_items"][varid] += qty
    request.session.modified = True

    count = len(request.session["cart"]["cart_items"].keys())
    return JsonResponse({"success": True, "count": count})


def getCart(request):
    if not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False):
        return JsonResponse({"success": False})
    cart = request.session["cart"]["cart_items"]

    try:
        currency = settings.CURRENCY
    except AttributeError:
        currency = "Rs."

    items = []
    total = 0
    for vid, qty in cart.items():
        try:
            vid = int(vid)
            v = Variation.objects.select_related("product").get(id=vid)
        except:
            pass
        else:
            k_v = []
            keyvalues = v.keyvalues.select_related("key", "value").all()
            for kv in keyvalues:
                k_v.append({
                    "key": kv.key.key.strip().lower().title(),
                    "value": kv.value.value.strip().lower().title()
                })
            
            price = float(v.price)
            amount = math.ceil(qty*price)
            items.append({
                "id": v.id,
                "price": round(float(v.price), 2),
                "title": v.product.title.strip().lower().title(),
                "variation_url": reverse("store:variation", kwargs={"product_slug": v.product.slug, "variation_id": v.id}),
                "unit": v.unit.strip().lower().title(),
                "keyvalues": k_v,
                "qty": qty,
                "amount": amount,
                "currency": currency
            })
            total += amount
    
    discount = False
    request.session["cart"]["cart_amount"] = total
    request.session.modified = True
    if request.session["cart"].get("cart_discount", False):
        dp = request.session["cart"]["cart_discount"]["percent"]
        discounted_amount = math.floor((float(total*dp))/100)
        amount_payable = total - discounted_amount
        if request.session["cart"]["cart_discount"]["discounted_amount"] != discounted_amount:
            request.session["cart"]["cart_discount"]["discounted_amount"] = discounted_amount
            request.session["cart"]["cart_discount"]["amount_payable"] = amount_payable
            request.session.modified = True
        discount = request.session["cart"]["cart_discount"]
        
    checkout_url = reverse("store:checkout")
    count = len(request.session["cart"]["cart_items"].keys())
    delivery_address = None if not request.session["cart"].get("delivery_address", False) else request.session["cart"]["delivery_address"]
    return JsonResponse({
        "success": True,
        "cart": items,
        "total": total,
        "currency": currency,
        "discount": discount,
        "checkout_url": checkout_url,
        "items_count": count,
        "delivery_address": delivery_address 
    })
    

def removeFromCart(request):
    if request.method == "GET":
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    vid = request.POST["vid"]
    if not vid:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        vid = int(vid)
        v = Variation.objects.get(id=vid)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    varid = str(vid)
    if not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False) \
        or varid not in request.session["cart"]["cart_items"]:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    qty = request.session["cart"]["cart_items"].pop(varid)

    if not request.session["cart"].get("cart_amount", False):
        request.session["cart"]["cart_amount"] = get_cart_amount(request)
    else:
        price = float(v.price)
        amount = math.ceil(qty*price)
        request.session["cart"]["cart_amount"] -= amount
    
    request.session.modified = True
    count = len(request.session["cart"]["cart_items"].keys())
    amount = request.session["cart"]["cart_amount"]

    if not count or not amount:
        request.session["cart"].clear()
        request.session.pop("cart", False)
        return JsonResponse({"success": True, "cart": False})
    
    if request.session["cart"].get("cart_discount", False):
        dp = request.session["cart"]["cart_discount"]["percent"]
        discounted_amount = math.floor((float(amount*dp))/100)
        amount_payable = amount - discounted_amount
        request.session["cart"]["cart_discount"]["discounted_amount"] = discounted_amount
        request.session["cart"]["cart_discount"]["amount_payable"] = amount_payable
        request.session.modified = True
        discount = request.session["cart"]["cart_discount"]
    else:
        discount = False

    return JsonResponse({"success": True, "count": count, "amount": amount, "cart": True, "discount": discount})


def get_cart_amount(request):
    if not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False):
        return 0
    
    total = 0
    for vid, qty in request.session["cart"]["cart_items"].items():
        try:
            varid = int(vid)
            v = Variation.objects.get(id=varid)
        except:
            pass
        else:
            price = float(v.price)
            amount = math.ceil(qty*price)
            total += amount
    
    return total


def apply_discount(request):
    if request.method == "GET" or not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False):
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    code = request.POST["code"]
    if not code:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        discount = Discount.objects.get(code=code)
    except:
        return JsonResponse({"success": False, "message": "Invalid Code"})
    
    if not request.session["cart"].get("cart_amount", False):
        request.session["cart"]["cart_amount"] = get_cart_amount(request)
        request.session.modified = True
    
    amount = request.session["cart"]["cart_amount"]
    dp = discount.percent
    discounted_amount = math.floor((float(amount*dp))/100)
    amount_payable = amount - discounted_amount
    
    try:
        currency = settings.CURRENCY
    except AttributeError:
        currency = "Rs."

    request.session["cart"]["cart_discount"] = {
        "code": code,
        "percent": discount.percent,
        "discounted_amount": discounted_amount,
        "amount_payable": amount_payable,
        "currency": currency
    }
    request.session.modified = True
    return JsonResponse({"success": True, "discount": request.session["cart"]["cart_discount"]})
    

def clearCart(request):
    if request.session.get("cart", False):
        request.session["cart"].clear()
        request.session.pop("cart", False)
    return JsonResponse({"success": True})


@login_required
@ensure_csrf_cookie
def checkout(request):
    if not request.session.get("cart", False):
        return HttpResponseRedirect(reverse("store:homepage"))
    
    return render(
        request,
        "store/checkout.html"
    )


@login_required
def getUserAddress(request):
    addresses = request.user.person.address.all()
    if not addresses or len(addresses) == 0:
        return JsonResponse({"success": False})
    
    a = []
    for address in addresses:
        a.append({
            "id": address.id,
            "first_name": address.first_name,
            "last_name": address.last_name,
            "email": address.email,
            "mobile": address.mobile,
            "homephone": address.homephone,
            "address1": address.address1,
            "address2": address.address2,
            "landmark": address.landmark,
            "pincode": address.pincode,
            "city": address.city,
            "state": address.state,
            "country": address.country
        })
    
    return JsonResponse({"success": True, "address": a})


@login_required
def addDeliveryAddress2Cart(request):
    if request.method == "GET" or not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False):
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    address_id = request.POST["address_id"]
    if not address_id:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        address_id = int(address_id)
        address = Address.objects.get(id=address_id)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if address.person.user != request.user:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    request.session["cart"]["delivery_address"] = {
        "id": address.id,
        "first_name": address.first_name,
        "last_name": address.last_name,
        "email": address.email,
        "mobile": address.mobile,
        "homephone": address.homephone,
        "address1": address.address1,
        "address2": address.address2,
        "landmark": address.landmark,
        "pincode": address.pincode,
        "city": address.city,
        "state": address.state,
        "country": address.country
    }
    request.session.modified = True
    return JsonResponse({"success": True, "delivery_address": request.session["cart"]["delivery_address"]})


@login_required
def removeSelectedDeliveryAddress2Cart(request):
    if request.method == "GET" or not request.session.get("cart", False) \
        or not request.session["cart"].get("cart_items", False) \
            or not request.session["cart"].get("delivery_address", False):
            return JsonResponse({"success": False, "message": "Invalid Request"})
    
    address_id = request.POST["address_id"]
    if not address_id:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        address_id = int(address_id)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if request.session["cart"]["delivery_address"]["id"] != address_id:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    request.session["cart"]["delivery_address"].clear()
    request.session["cart"].pop("delivery_address", False)
    request.session.modified = True
    return JsonResponse({"success": True})


@login_required
def addNewDeliveryAddress2Cart(request):
    if request.method == "GET" or not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False):
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    first_name = request.POST["first_name"]
    last_name = request.POST["last_name"]
    mobile = request.POST["mobile"]
    homephone = request.POST["homephone"]
    address1 = request.POST["address1"]
    address2 = request.POST["address2"]
    city = request.POST["city"]
    pincode = request.POST["pincode"]
    state = request.POST["state"]
    country = request.POST["country"]
    landmark = request.POST["landmark"]
    email = request.POST["email"]

    if not first_name or not last_name or not mobile or not address1 or not city \
        or not pincode or not state or not country or not landmark or not email:
        return JsonResponse({"success": False, "message": "Incomplete Form"})
    
    if not validate_email(email):
        return JsonResponse({"success": False, "message": "Invalid Email"})
    
    address = Address.objects.create(
        person=request.user.person,
        first_name=first_name.strip().lower().title(),
        last_name=last_name.strip().lower().title(),
        email=email,
        mobile=mobile,
        homephone=None if not homephone else homephone,
        address1=address1.strip().lower().title(),
        address2=None if not address2 else address2.strip().lower().title(),
        landmark=landmark.strip().lower().title(),
        pincode=pincode.strip(),
        city=city.strip().lower().title(),
        state=state.strip().lower().title(),
        country=country.strip().lower().title()
    )

    request.session["cart"]["delivery_address"] = {
        "id": address.id,
        "first_name": address.first_name,
        "last_name": address.last_name,
        "email": address.email,
        "mobile": address.mobile,
        "homephone": address.homephone,
        "address1": address.address1,
        "address2": address.address2,
        "landmark": address.landmark,
        "pincode": address.pincode,
        "city": address.city,
        "state": address.state,
        "country": address.country
    }
    request.session.modified = True
    return JsonResponse({"success": True, "delivery_address": request.session["cart"]["delivery_address"]})


@login_required
def place_order(request):
    if not request.session.get("cart", False) or not request.session["cart"].get("cart_items", False) \
        or not request.session["cart"].get("delivery_address", False) or request.method == "GET":
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    time = request.POST["time"]
    date = request.POST["date"]
    info = request.POST["info"]

    if info and len(info) > 200:
        return JsonResponse({"success": False, "message": "Maxinum 200 characters allowed."})

    if date and time:
        time_data = str(date) + ' ' + str(time)
        format_data = "%Y-%m-%d %H:%M"
        try:
            preferred_date = datetime.strptime(time_data, format_data)
        except:
            preferred_date = None
    elif date:
        time_data = str(date)
        format_data = "%Y-%m-%d"
        try:
            preferred_date = datetime.strptime(time_data, format_data)
        except:
            preferred_date = None
    else:
        preferred_date = None
    
    try:
        delivery_time_hours = settings.DELIVERY_TIME_IN_HOURS
    except AttributeError:
        delivery_time_hours = 24
    
    try:
        delivery_time_days = settings.DELIVERY_TIME_IN_DAYS
    except AttributeError:
        delivery_time_days = 1
    
    if preferred_date and (preferred_date - datetime.now()).days < delivery_time_days:
        return JsonResponse({"success": False, "message": f"We cannot deliver this product in less than {delivery_time_days} business day(s)."})

    address_id = request.session["cart"]["delivery_address"]["id"]
    try:
        address = Address.objects.get(id=address_id)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if address.person.user != request.user:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    discount = None
    amount_payable = request.session["cart"]["cart_amount"]
    discount_code = None if not request.session["cart"].get("cart_discount", False) else request.session["cart"]["cart_discount"]["code"]
    if discount_code:
        try:
            discount = Discount.objects.get(code=discount_code)
        except:
            pass
        else:
            amount_payable = request.session["cart"]["cart_discount"]["amount_payable"]
    
    try:
        with transaction.atomic():
            order = Order.objects.create(
                customer = request.user.person,
                address = address,
                amount = request.session["cart"]["cart_amount"],
                discount = discount,
                amount_payable = amount_payable,
                preferred_delivery_time = preferred_date,
                additional_info = None if not info else info,
                count = len(request.session["cart"]["cart_items"].keys())
            )

            for vid, qty in request.session["cart"]["cart_items"].items():
                vid = int(vid)
                v = Variation.objects.get(id=vid)
                order_item = OrderItem.objects.create(
                    order = order,
                    product = v,
                    quantity = qty,
                    unit_price = v.price,
                    amount = math.ceil(float(v.price)*qty)
                )
    except:
        request.session["cart"].clear()
        request.session.pop("cart", False)
        return_url = reverse('store:homepage')
        messages.add_message(
            request,
            messages.ERROR,
            "Something went wrong. Please try again later."
        )
        return JsonResponse({"success": True, "return_url": return_url})
    
    request.session["cart"].clear()
    request.session.pop("cart", False)
    
    messages.add_message(
        request,
        messages.SUCCESS,
        "Thank you for your order."
    )
    return_url = reverse("store:order", kwargs={'order_id': order.id})
    return JsonResponse({"success": True, "return_url": return_url})


@login_required
@ensure_csrf_cookie
def orders(request):
    orders = get_orders(request)
    return render(
        request,
        "store/orders.html",
        {"orders": orders}
    )


@login_required
@ensure_csrf_cookie
def order(request, order_id):
    order_details = get_order_details(request, order_id)
    if not order_details:
        return HttpResponseRedirect(reverse("store:orders"))
    return render(
        request,
        'store/order.html',
        {"order": order_details}
    )


@login_required
def cancelOrder(request):
    if request.method == "GET":
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    order_id = request.POST["order_id"]
    reason = request.POST["reason"]

    if not order_id:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    if reason and len(reason) > 200:
        return JsonResponse({"success": False, "message": "Maxinum 200 characters allowed."})
    
    try:
        order_id = int(order_id)
        order = Order.objects.get(id=order_id)
    except:
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    if order.customer != request.user.person or order.status != 'O':
        return JsonResponse({"success": False, "message": "Invalid Request"})
    
    try:
        with transaction.atomic():
            oc = OrderCancellation.objects.create(
                order = order,
                reason = None if not reason else reason
            )
            order.status = 'P'
            order.save()
    except:
        return JsonResponse({"success": False, "message": "Something went wrong. Please try again later."})
    return JsonResponse({"success": True})


@login_required
def reopenOrder(request, order_id):
    try:
        order_id = int(order_id)
        order = Order.objects.get(id=order_id)
    except:
        return HttpResponseRedirect(reverse("store:orders"))
    
    if order.customer != request.user.person:
        return HttpResponseRedirect(reverse("store:orders"))
    
    if order.status != 'P':
        messages.add_message(
            request,
            messages.INFO,
            "You cannot reopen this order. Kindly place a fresh order."
        )
        return HttpResponseRedirect(reverse("store:order", kwargs={'order_id': order_id}))
    
    try:
        delivery_time_days = settings.DELIVERY_TIME_IN_DAYS
    except AttributeError:
        delivery_time_days = 1
    
    try:
        with transaction.atomic():
            if order.preferred_delivery_time and (order.preferred_delivery_time - datetime.now(timezone.utc)).days < delivery_time_days:
                order.preferred_delivery_time = None
            
            order.status = 'O'
            order.placed_at = datetime.now()
            order.save()
            order.cancellation.delete()
    except:
        messages.add_message(
            request,
            messages.ERROR,
            "Something went wrong. Please try again later."
        )
        return HttpResponseRedirect(reverse("store:order", kwargs={'order_id': order_id}))
    
    messages.add_message(
        request,
        messages.SUCCESS,
        "Your order is active."
    )
    return HttpResponseRedirect(reverse("store:order", kwargs={'order_id': order_id}))


def search(request):
    q = request.GET.get('q')
    if not q:
        return HttpResponseRedirect(reverse("store:homepage"))
    
    products = get_search_products(request, q)
    return render(
        request,
        "store/search.html",
        {"products": products, "search": True, "q": q}
    )


@login_required
def database_update(request):
    if request.user.is_superuser:
        database.UPLOAD_CATEGORIES()
        database.UPLOAD_PRODUCTS()
        database.UPLOAD_TAGS()
        database.UPDATE_PROUCTS()
        database.UPDATE_KEYVALUE()
        database.UPDATE_VARIATIONS()
        database.inventory()
    return HttpResponseRedirect(reverse("store:homepage"))


@login_required
def clear_database_but_images(request):
    if request.user.is_superuser:
        database.clear_all_but_images()
    return HttpResponseRedirect(reverse("store:homepage"))


@login_required
def clear_database_images(request):
    if request.user.is_superuser:
        database.clear_images()
    return HttpResponseRedirect(reverse("store:homepage"))


# Generated by Django 4.1 on 2022-09-09 11:43

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('authentication', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField(blank=True, max_length=300, null=True, unique=True)),
            ],
            options={
                'verbose_name_plural': 'Categories',
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='Discount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True)),
                ('percent', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(max_length=1000, unique=True, upload_to='static/images/product-images/')),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='Key',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=64, unique=True)),
            ],
            options={
                'ordering': ['key'],
            },
        ),
        migrations.CreateModel(
            name='KeyValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='store.key')),
            ],
            options={
                'verbose_name_plural': 'Key-Value-Pairs',
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, validators=[django.core.validators.MinValueValidator(1)])),
                ('amount_payable', models.DecimalField(decimal_places=2, max_digits=20, validators=[django.core.validators.MinValueValidator(1)])),
                ('status', models.CharField(choices=[('O', 'OPEN'), ('C', 'CLOSED'), ('N', 'CANCELLED'), ('P', 'PENDING_CANCELLATION'), ('F', 'FAILED')], default='O', max_length=1)),
                ('placed_at', models.DateTimeField(auto_now_add=True)),
                ('closed_at', models.DateTimeField(blank=True, null=True)),
                ('cancelled_at', models.DateTimeField(blank=True, null=True)),
                ('preferred_delivery_time', models.DateTimeField(blank=True, null=True)),
                ('additional_info', models.CharField(blank=True, max_length=1000, null=True)),
                ('count', models.IntegerField(default=1)),
                ('address', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='authentication.address')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='orders', to='authentication.person')),
                ('discount', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='orders', to='store.discount')),
            ],
            options={
                'ordering': ['-placed_at'],
            },
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
                ('info', models.TextField(blank=True, max_length=1000, null=True)),
                ('categories', models.ManyToManyField(blank=True, related_name='products', to='store.category')),
                ('images', models.ManyToManyField(blank=True, related_name='products', to='store.image')),
            ],
            options={
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Value',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=64, unique=True)),
            ],
            options={
                'ordering': ['value'],
            },
        ),
        migrations.CreateModel(
            name='Variation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.DecimalField(decimal_places=2, default=1, max_digits=20, validators=[django.core.validators.MinValueValidator(1)])),
                ('unit', models.CharField(default='Kg', max_length=10)),
                ('availability', models.BooleanField(default=True)),
                ('info', models.TextField(blank=True, max_length=1000, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('inventory', models.IntegerField(default=0)),
                ('images', models.ManyToManyField(blank=True, related_name='variations', to='store.image')),
                ('keyvalues', models.ManyToManyField(blank=True, to='store.keyvalue')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='variations', to='store.product')),
            ],
            options={
                'ordering': ['product'],
            },
        ),
        migrations.AddField(
            model_name='product',
            name='tags',
            field=models.ManyToManyField(blank=True, related_name='products', to='store.tag'),
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('quantity', models.DecimalField(decimal_places=2, max_digits=6)),
                ('unit_price', models.DecimalField(decimal_places=2, max_digits=20)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='orderitems', to='store.order')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='orderitems', to='store.variation')),
            ],
        ),
        migrations.CreateModel(
            name='OrderCancellation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reason', models.CharField(blank=True, max_length=1000, null=True)),
                ('cancelled_at', models.DateTimeField(auto_now_add=True)),
                ('order', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='cancellation', to='store.order')),
            ],
            options={
                'ordering': ['-cancelled_at'],
            },
        ),
        migrations.AddField(
            model_name='keyvalue',
            name='value',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='store.value'),
        ),
        migrations.AddField(
            model_name='key',
            name='values',
            field=models.ManyToManyField(through='store.KeyValue', to='store.value'),
        ),
    ]



from django.db import models
from django.urls import reverse
from django.core.validators import MinValueValidator
from authentication.models import Person, Address


class Image(models.Model):
    image = models.ImageField(max_length=1000, unique=True, null=False, upload_to="static/images/product-images/")
    name = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.name}"
    
    class Meta:
        ordering = ["id"]


class Tag(models.Model):
    name = models.CharField(max_length=64, unique=True)

    def __str__(self):
        return f"{self.name}"
    
    class Meta:
        ordering = ["name"]


class Category(models.Model):
    title = models.CharField(max_length=255, unique=True, null=False)
    slug = models.SlugField(max_length=300, unique=True, null=True, blank=True)

    def __str__(self):
        return f"{self.title}"
    
    class Meta:
        ordering = ["title"]
        verbose_name_plural = "Categories"


class Product(models.Model):
    title = models.CharField(max_length=255, unique=True, null=False)
    slug = models.SlugField(max_length=255, blank=True, null=True)
    info = models.TextField(max_length=1000, blank=True, null=True)
    images = models.ManyToManyField(Image, blank=True, related_name="products")
    tags = models.ManyToManyField(Tag, blank=True, related_name="products")
    categories = models.ManyToManyField(Category, related_name="products", blank=True)

    def __str__(self):
        return f"{self.title}"
    
    class Meta:
        ordering = ["title"]


class Key(models.Model):
    key = models.CharField(max_length=64, unique=True)
    values = models.ManyToManyField("Value", through="KeyValue")

    def __str__(self):
        return f"{self.key}"
    
    class Meta:
        ordering = ["key"]


class Value(models.Model):
    value = models.CharField(max_length=64, unique=True)

    def __str__(self):
        return f"{self.value}"
    
    class Meta:
        ordering = ["value"]


class KeyValue(models.Model):
    key = models.ForeignKey(Key, on_delete=models.CASCADE)
    value = models.ForeignKey(Value, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.key.key} {self.value.value}"
    
    class Meta:
        verbose_name_plural = "Key-Value-Pairs"


class Variation(models.Model):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, related_name="variations")
    price = models.DecimalField(
        default=1,
        max_digits=20,
        decimal_places=2,
        validators=[MinValueValidator(1)]
    )
    unit = models.CharField(max_length=10, default="Kg")
    availability = models.BooleanField(default=True)
    info = models.TextField(max_length=1000, blank=True, null=True)
    keyvalues = models.ManyToManyField(KeyValue, blank=True)
    images = models.ManyToManyField(Image, blank=True, related_name="variations")
    created = models.DateTimeField(auto_now_add=True)
    inventory = models.IntegerField(default=0)

    def __str__(self):
        return f"{self.product.title} | Rs. {self.price}/-"
    
    class Meta:
        ordering = ["product"]

class Discount(models.Model):
    code = models.CharField(max_length=10, unique=True)
    percent = models.IntegerField()

    def __str__(self):
        return f"{self.code} - {self.percent}%"


class Order(models.Model):
    OPEN = 'O'
    CLOSED = 'C'
    CANCELLED = 'N'
    PENDING_CANCELLATION = 'P'
    FAILED = 'F'

    ORDER_STATUS_CHOICES = [
        (OPEN, "OPEN"),
        (CLOSED, "CLOSED"),
        (CANCELLED, "CANCELLED"),
        (PENDING_CANCELLATION, "PENDING_CANCELLATION"),
        (FAILED, "FAILED")
    ]

    customer = models.ForeignKey(Person, on_delete=models.PROTECT, related_name="orders")
    address = models.ForeignKey(Address, on_delete=models.PROTECT)
    amount = models.DecimalField(max_digits=20, decimal_places=2, validators=[MinValueValidator(1)])
    discount = models.ForeignKey(Discount, on_delete=models.PROTECT, blank=True, null=True, related_name="orders")
    amount_payable = models.DecimalField(max_digits=20, decimal_places=2, validators=[MinValueValidator(1)])
    status = models.CharField(max_length=1, choices=ORDER_STATUS_CHOICES, default=OPEN)
    placed_at = models.DateTimeField(auto_now_add=True)
    closed_at = models.DateTimeField(blank=True, null=True)
    cancelled_at = models.DateTimeField(blank=True, null=True)
    preferred_delivery_time = models.DateTimeField(blank=True, null=True)
    additional_info = models.CharField(max_length=1000, blank=True, null=True)
    count = models.IntegerField(default=1)

    def __str__(self):
        return f"{self.id}"
    
    class Meta:
        ordering = ["-placed_at"]


class OrderItem(models.Model):
    order = models.ForeignKey(Order, on_delete=models.PROTECT, related_name="orderitems")
    product = models.ForeignKey(Variation, on_delete=models.PROTECT, related_name="orderitems")
    quantity = models.DecimalField(max_digits=6, decimal_places=2)
    unit_price = models.DecimalField(max_digits=20, decimal_places=2)
    amount = models.DecimalField(max_digits=20, decimal_places=2)


class OrderCancellation(models.Model):
    order = models.OneToOneField(Order, on_delete=models.PROTECT, related_name="cancellation")
    reason = models.CharField(max_length=1000, blank=True, null=True)
    cancelled_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ["-cancelled_at"]


from decimal import Decimal
from .models import Key, KeyValue, Order, OrderCancellation, OrderItem, Product, Variation, Image, Tag, Category, Value, Discount
import random
from django.utils.text import slugify

INFO = "Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged."

def UPLOAD_CATEGORIES():
    categories = [
        {"category":"Outdoors"},
        {"category":"Movies"},
        {"category":"Beauty"},
        {"category":"Beauty"},
        {"category":"Outdoors"},
        {"category":"Movies"},
        {"category":"Home"},
        {"category":"Movies"},
        {"category":"Sports"},
        {"category":"Jewelry"},
        {"category":"Computers"},
        {"category":"Grocery"},
        {"category":"Clothing"},
        {"category":"Electronics"},
        {"category":"Toys"},
        {"category":"Grocery"},
        {"category":"Automotive"},
        {"category":"Music"},
        {"category":"Outdoors"},
        {"category":"Books"},
        {"category":"Health"},
        {"category":"Computers"},
        {"category":"Sports"},
        {"category":"Sports"},
        {"category":"Baby"},
        {"category":"Beauty"},
        {"category":"Home"},
        {"category":"Grocery"},
        {"category":"Jewelry"},
        {"category":"Electronics"},
        {"category":"Automotive"},
        {"category":"Automotive"},
        {"category":"Electronics"},
        {"category":"Games"},
        {"category":"Tools"},
        {"category":"Electronics"},
        {"category":"Shoes"},
        {"category":"Industrial"},
        {"category":"Toys"},
        {"category":"Industrial"},
        {"category":"Music"},
        {"category":"Shoes"},
        {"category":"Grocery"},
        {"category":"Books"},
        {"category":"Automotive"},
        {"category":"Sports"},
        {"category":"Tools"},
        {"category":"Games"},
        {"category":"Books"},
        {"category":"Sports"}
    ]
    c = set()
    for category in categories:
        c.add(category["category"])
    
    for category in c:
        ct = Category(
            title=category,
            slug=slugify(category),
        )
        ct.save()


def UPLOAD_PRODUCTS():
    products = [
        {"title":"Potatoes - Parissienne"},
        {"title":"Wine - Dubouef Macon - Villages"},
        {"title":"Pastry - Mini French Pastries"},
        {"title":"Muffin - Carrot Individual Wrap"},
        {"title":"Kohlrabi"},
        {"title":"Gin - Gilbeys London, Dry"},
        {"title":"Flower - Daisies"},
        {"title":"Orange - Tangerine"},
        {"title":"Broom Handle"},
        {"title":"Sauce - Hollandaise"},
        {"title":"Salt - Table"},
        {"title":"Lid - 10,12,16 Oz"},
        {"title":"Pepper - Black, Whole"},
        {"title":"Bread - Onion Focaccia"},
        {"title":"Banana - Green"},
        {"title":"Ice Cream Bar - Rolo Cone"},
        {"title":"Sauce - Rosee"},
        {"title":"Transfer Sheets"},
        {"title":"Muffin - Zero Transfat"},
        {"title":"Champagne - Brights, Dry"},
        {"title":"Bread - 10 Grain Parisian"},
        {"title":"Nut - Walnut, Pieces"},
        {"title":"Muffin Batt - Ban Dream Zero"},
        {"title":"Vinegar - Cider"},
        {"title":"Soup - Campbells, Beef Barley"},
        {"title":"Table Cloth 53x69 White"},
        {"title":"Sorrel - Fresh"},
        {"title":"Beef - Rouladin, Sliced"},
        {"title":"Nantucket - Carrot Orange"},
        {"title":"Wine - Sherry Dry Sack, William"},
        {"title":"Food Colouring - Green"},
        {"title":"Beef - Bresaola"},
        {"title":"Lettuce - Green Leaf"},
        {"title":"Cake Circle, Paprus"},
        {"title":"Juice - Apple, 341 Ml"},
        {"title":"Rambutan"},
        {"title":"Coffee Cup 12oz 5342cd"},
        {"title":"Yukon Jack"},
        {"title":"Beer - Alexander Kieths, Pale Ale"},
        {"title":"Milk - 2%"},
        {"title":"Langers - Mango Nectar"},
        {"title":"Muffin Batt - Carrot Spice"},
        {"title":"Flour - Corn, Fine"},
        {"title":"Wine - Balbach Riverside"},
        {"title":"Huck White Towels"},
        {"title":"Asparagus - Green, Fresh"},
        {"title":"Guava"},
        {"title":"Juice - Apple, 1.36l"},
        {"title":"Pepper - Cayenne"},
        {"title":"Pastry - Baked Cinnamon Stick"},
        {"title":"Bread - Raisin"},
        {"title":"Barley - Pearl"},
        {"title":"Taro Leaves"},
        {"title":"Chicken Breast Halal"},
        {"title":"Wine - Chardonnay South"},
        {"title":"Mustard - Dry, Powder"},
        {"title":"Flour - So Mix Cake White"},
        {"title":"Melon - Cantaloupe"},
        {"title":"Chip - Potato Dill Pickle"},
        {"title":"Pate Pans Yellow"},
        {"title":"Poppy Seed"},
        {"title":"Beef - Kobe Striploin"},
        {"title":"Appetizer - Escargot Puff"},
        {"title":"Coconut - Creamed, Pure"},
        {"title":"Pail - 4l White, With Handle"},
        {"title":"Wine - Taylors Reserve"},
        {"title":"Bulgar"},
        {"title":"Beer - Moosehead"},
        {"title":"Soup - Cream Of Broccoli, Dry"},
        {"title":"Piping - Bags Quizna"},
        {"title":"Lettuce - Red Leaf"},
        {"title":"Silicone Parch. 16.3x24.3"},
        {"title":"Tomatoes - Diced, Canned"},
        {"title":"Muffin Orange Individual"},
        {"title":"Rum - Spiced, Captain Morgan"},
        {"title":"Fish - Artic Char, Cold Smoked"},
        {"title":"Cabbage - Green"},
        {"title":"Flour - Bran, Red"},
        {"title":"Cookies Cereal Nut"},
        {"title":"Table Cloth 81x81 Colour"},
        {"title":"Veal - Insides Provini"},
        {"title":"Soup - Knorr, Chicken Gumbo"},
        {"title":"Ice Cream - Turtles Stick Bar"},
        {"title":"Vacuum Bags 12x16"},
        {"title":"Bread - Assorted Rolls"},
        {"title":"Salsify, Organic"},
        {"title":"Turkey - Breast, Bone - In"},
        {"title":"Cup - 3.5oz, Foam"},
        {"title":"Veal - Shank, Pieces"},
        {"title":"Mace"},
        {"title":"Sausage - Andouille"},
        {"title":"C - Plus, Orange"},
        {"title":"Tarragon - Primerba, Paste"},
        {"title":"Pineapple - Canned, Rings"},
        {"title":"Grapes - Black"},
        {"title":"Milk - 1%"},
        {"title":"Coffee - Hazelnut Cream"},
        {"title":"Fish - Bones"},
        {"title":"Oregano - Dry, Rubbed"},
        {"title":"Lid - Oz"}
    ]

    p = set()
    for product in products:
        p.add(product["title"])
    
    for product in p:
        pd = Product(
            title=product,
            slug=slugify(product),
            info="Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged."
        )
        pd.save()


def UPLOAD_TAGS():
    tags = [
        {"tag":"yfiiew"},
        {"tag":"zicpwt"},
        {"tag":"keoewn"},
        {"tag":"bjztfz"},
        {"tag":"ucqgwv"},
        {"tag":"stneve"},
        {"tag":"yuexut"},
        {"tag":"lepsbk"},
        {"tag":"usfgki"},
        {"tag":"vrnqcn"},
        {"tag":"gqulmk"},
        {"tag":"hotjmr"},
        {"tag":"xmytvs"},
        {"tag":"vwkbxf"},
        {"tag":"eiqzzz"},
        {"tag":"mxeoic"},
        {"tag":"jgicpc"},
        {"tag":"xmvcgn"},
        {"tag":"asxyhw"},
        {"tag":"mszrlm"},
        {"tag":"sxnrjd"},
        {"tag":"veuxfu"},
        {"tag":"ceriaq"},
        {"tag":"ubwvlf"},
        {"tag":"jntmxu"},
        {"tag":"goaava"},
        {"tag":"szyqrj"},
        {"tag":"khwuzf"},
        {"tag":"lfzslz"},
        {"tag":"mbpyhk"},
        {"tag":"ckfcth"},
        {"tag":"zvicje"},
        {"tag":"qlappt"},
        {"tag":"xznjnf"},
        {"tag":"quqjvs"},
        {"tag":"noewsk"},
        {"tag":"kenbpi"},
        {"tag":"yqclak"},
        {"tag":"zjhtms"},
        {"tag":"oujsjn"},
        {"tag":"csynvf"},
        {"tag":"rshgly"},
        {"tag":"lbjfqh"},
        {"tag":"zxegcw"},
        {"tag":"ejlsrp"},
        {"tag":"twsaqo"},
        {"tag":"ocfoji"},
        {"tag":"uvubrg"},
        {"tag":"nggqwq"},
        {"tag":"wyooor"}
    ]

    t = set()
    for tag in tags:
        t.add(tag["tag"])
    
    for tag in t:
        tg = Tag(
            name=tag
        )
        tg.save()


def UPDATE_PROUCTS():    
    images = Image.objects.all()
    tags = Tag.objects.all()
    categories = Category.objects.all()

    products = Product.objects.all()
    for product in products:
        a, b = 1, 1
        while a == b:
            a = random.randint(0, 6)
            b = random.randint(0, 6)
        product.images.add(images[a], images[b])
        
        a, b = 1, 1
        while a == b:
            a = random.randint(0, 19)
            b = random.randint(0, 19)
        product.categories.add(categories[a], categories[b])

        a, b = 1, 1
        while a == b:
            a = random.randint(0, 49)
            b = random.randint(0, 49)
        product.tags.add(tags[a], tags[b])
        product.save()
    return


def UPDATE_KEYVALUE():
    companies = [
        {"tag":"Ainyx"},
        {"tag":"Facebook"},
        {"tag":"Twitter"},
        {"tag":"Voomm"},
        {"tag":"Google"},
        {"tag":"Feedfire"},
        {"tag":"Microsoft"},
        {"tag":"Muxo"}
    ]
    k = Key(key="Company")
    k.save()

    c = set()
    for company in companies:
        c.add(company["tag"])
    
    for company in c:
        v = Value(value=company)
        v.save()
        
        kv = KeyValue(key=k, value=v)
        kv.save()

    brands = [
        {"tag":"OXYGEN"},
        {"tag":"Epithelium"},
        {"tag":"RIDAURA"},
        {"tag":"SEROQUEL"},
        {"tag":"HOUSE DUST"},
        {"tag":"Hydrochlorothiazide"},
        {"tag":"Nicardipine"},
        {"tag":"Naproxen Sodium"},
        {"tag":"Hydrocortisone"},
        {"tag":"Methylphenidate"}
    ]

    k = Key(key="Brand")
    k.save()

    c = set()
    for brand in brands:
        c.add(brand["tag"])
    
    for brand in c:
        v = Value(value=brand)
        v.save()
        
        kv = KeyValue(key=k, value=v)
        kv.save()

    colors = [
        {"tag":"Fuscia"},
        {"tag":"Teal"},
        {"tag":"Green"},
        {"tag":"Mauv"},
        {"tag":"Turquoise"},
        {"tag":"Fuscia"},
        {"tag":"Turquoise"},
        {"tag":"Aquamarine"},
        {"tag":"Goldenrod"},
        {"tag":"Pink"}
    ]

    k = Key(key="Color")
    k.save()

    c = set()
    for color in colors:
        c.add(color["tag"])
    
    for color in c:
        v = Value(value=color)
        v.save()
        
        kv = KeyValue(key=k, value=v)
        kv.save()


def UPDATE_VARIATIONS():
    keyvalues = KeyValue.objects.all()
    images = Image.objects.all()

    products = Product.objects.all()
    for product in products:
        for _ in range(3):
            v = Variation(
                product=product,
                price=Decimal(random.randint(200, 500)),
                availability=True,
                info=INFO
            )
            v.save()

            a, b, c = 1, 1, 1
            while a == b or a == c or b == c:
                a = random.randint(0, 25)
                b = random.randint(0, 25)
                c = random.randint(0, 25)
            v.keyvalues.add(keyvalues[a], keyvalues[b], keyvalues[c])
    
            a, b = 1, 1
            while a == b:
                a = random.randint(0, 6)
                b = random.randint(0, 6)
            v.images.add(images[a], images[b])
    return


def clear_all_but_images():
    oc = OrderCancellation.objects.all().delete()
    odi = OrderItem.objects.all().delete()
    o = Order.objects.all().delete()
    d = Discount.objects.all().delete()
    v = Variation.objects.all().delete()
    kv = KeyValue.objects.all().delete()
    v = Value.objects.all().delete()
    k = Key.objects.all().delete()
    p = Product.objects.all().delete()
    c = Category.objects.all().delete()
    t = Tag.objects.all().delete()


def clear_images():
    i = Image.objects.all().delete()


def inventory():
    variations = Variation.objects.all()
    for variation in variations:
        variation.inventory = random.randint(0, 50)
        variation.save()
    return
